module vga
(
  input clock,
  output r, output g, output b,
  output hs, output vs,
  output reg [11:0] address,
  input      [ 7:0] data
);

reg [9:0] x = 10'b0;
reg [8:0] y = 9'b0;

wire xborder = x == 10'd799;
wire yborder = y ==  9'd448;
wire visible = x >= 48 && x < 48+640 && y >= 35 && y <= 35+400;

// Горизонтальная и вертикальная синхронизация
assign hs = x  < 48+640+16;
assign vs = y >= 35+400+12;

// Выравнивание
wire [9:0] xv = x - 40; // Не 48, а 40, из-за конвейерного метода
wire [8:0] yv = y - 35; // А здесь 35

// Временные регистры
reg [7:0] rmask;   // Маска из знакогенератора
reg       tcolor;  // Здесь хранится временное значение цвета
reg       rcolor;  // Цвет из символа (из tchar)

// Итоговый пиксель
wire      pix = rmask[ ~xv[2:0] ] ^ rcolor;

// Вывод белого экрана
assign {r, g, b} = visible & pix ? 3'b111 : 3'b000;

always @(posedge clock) begin

    x <= xborder ? 1'b0 : x + 1'b1;
    y <= yborder ? 1'b0 : (xborder ? y + 1'b1 : y);

    /* verilator lint_off CASEINCOMPLETE */
    case (xv[2:0])

    3'h0: begin address <= xv[9:3] + yv[8:4]*80; end
    3'h1: begin address <= {1'b1, data[6:0], yv[3:0]}; tcolor <= data[7]; end
    3'h7: begin rmask   <= data; rcolor <= tcolor; end

    endcase

end

endmodule
